% Plot Sector Inflation


clear all;
small = 1.0e-10;
big = 1.0e+6;                  
%rng(63761);
 
 % -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
  
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_real;
  calvec_r = calvec_qr;
  calds_r = calds_qr;
  dnobs_r = dnobs_qr;
  
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Real Variables
  yvar = 0.5*log(gdp)+0.5*log(gdi); labelvec_r = {'GDO'};
  yvar = [yvar capu]; labelvec_r = [labelvec_r;'Capacity utilization'];
  yvar = [yvar log(emp_est)]; labelvec_r = [labelvec_r;'Employment (establishments)'];
  yvar = [yvar emp_pop]; labelvec_r = [labelvec_r;'Employment/Population'];
  yvar = [yvar -urate]; labelvec_r = [labelvec_r;'Unemployment rate (inverted)'];
  yvar = [yvar -urate_st]; labelvec_r = [labelvec_r;'Short term unemployment rate (inverted)'];

  % Lags used in PC
  pc_lag = [2 2 0 0 0 2]';

  % List variables used and lags
  fprintf('Using real variables: \n');
  for i = 1:size(yvar,2);
        str = char(labelvec_r(i));
        str = [str '  lags =' num2str(pc_lag(i)) '\n'];
        fprintf(str);
  end;

  yvar_bp = NaN(size(yvar));
  for i = 1:size(yvar,2);
      [xbpadj tmp] = bpassar(yvar(:,i),updc,lpdc,n,nar,tcode,seflag);
      yvar_bp(:,i) = xbpadj;
  end;
  
  % Form variable matrix for PC;
  ypc_data = NaN(size(yvar_bp));
  jj = 1;
  for i = 1:size(yvar,2);
        ypc_data(:,jj) = lag(yvar_bp(:,i),pc_lag(i));
        jj = jj+1;
  end;

 first_date = [1984 1];
 last_date = [2018 1];
 ismpl_r = smpl(calvec_r,first_date,last_date,4);
 calvec = calvec_r(ismpl_r==1);
 n_t = size(calvec,1);

 % Construct PC
 ypc_data = ypc_data(ismpl_r==1,:);
 ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
 [pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
 ybp_pc = pc_score(:,1);
 r2 = cumsum(pc_latent)/sum(pc_latent);
 fprintf('Trace R2 of first PC: %5.2f \n',r2(1));
 ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);

  % Plot Level of Disaggregate Inflation
  ax_fs = 15;
  figure;
  pan = {'a';'b';'c';'d';'e';'f'};
  for i = 1:6;
      ss = char(labelvec_r(i));
      subplot(3,2,i);
      plot(calvec,ypc_data_normalized(:,6),'-- b','LineWidth',2);
      hold on;
        plot(calvec,ypc_data_normalized(:,i),'- k','LineWidth',3);
      hold off;
      if i == 1
         legend('Short term unemployment rate (inverted)','Cyclical activity measure','Location','north');
      end
      xlim([1980 2020]);
      title(ss);
      ax = gca;
      ax.FontSize = ax_fs;
      
      % Save Values to CSV File
      ss = char(pan(i));
      outfile_name = [outdir 'Chart_3_2_' ss '.csv'];
      fileID = fopen(outfile_name,'w');
      fprintf(fileID,'date,activity measure,st urate \n');
      tmp = [calvec ypc_data_normalized(:,i) ypc_data_normalized(:,6)];
      prtmat_comma(tmp,fileID,'%12.4f','\n');
      
  end;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_3_2']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
  % Plot PC and output measures
  figure;
  ax_fs = 25;
  plot(calvec,ybp_pc,'- k','LineWidth',4);
  hold on;
    plot(calvec,ypc_data_normalized(:,1),'-- b','LineWidth',1);
    plot(calvec,ypc_data_normalized(:,2),': b','LineWidth',1);
    plot(calvec,ypc_data_normalized(:,3),'-. b','LineWidth',1);
    plot(calvec,ypc_data_normalized(:,4),'-- r','LineWidth',1);
    plot(calvec,ypc_data_normalized(:,5),': r','LineWidth',1);
    plot(calvec,ypc_data_normalized(:,6),'-. r','LineWidth',1);
  hold off;
  legend('Cyclical Activity Index','GDO (lagged 2 quarters)','Capacity utilization (lagged 2 quarters)','Employment','Employment/Population','Unemployment rate (inverted)','Short term unemp. rate (inverted, lagged 2 quarters)');
  xlim([1980 2020]);
  ax = gca;
  ax.FontSize = ax_fs;
  
  % Save Values to CSV File
  ss = char(pan(i));
  outfile_name = [outdir 'Chart_3_3.csv'];
  fileID = fopen(outfile_name,'w');
  fprintf(fileID,'date, Cyclical Activity Index, GDO (lagged 2 quarters), Capacity utilization (lagged 2 quarters), Employment, Employment/Population, Unemployment rate (inverted,Short term unemp. rate (inverted, lagged 2 quarters) \n');
  tmp = [calvec ybp_pc ypc_data_normalized(:,1:6)];
  prtmat_comma(tmp,fileID,'%12.4f','\n');
  
  
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_3_3']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
  
  path(p);  % Reset path
  
   
  

  