% Plot Sector Inflation


clear all;
small = 1.0e-10;
big = 1.0e+6;                  
%rng(63761);
 
 % -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
  
  
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_prices;
  % Data Series Used
  dp_agg = dp_agg_q;
  dp_agg_xfe = dp_agg_xfe_q;
  dp_agg_xe = dp_agg_xe_q;
  dp_disagg = dp_disagg_q;
  share_avg = share_avg_q;
  share_avg_xe = share_avg_xe_q;
  share_avg_xfe= share_avg_xfe_q;
  calvec_p = calvec_qp;
  dnobs_p = dnobs_qp;
  calds_p = calds_qp;
  dp_aggs = [dp_agg dp_agg_xfe dp_agg_xe];
  namevec_aggs = {'dp_agg';'dp_agg_xfe';'dp_agg_xe'};
  
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_real;
  calvec_r = calvec_qr;
  calds_r = calds_qr;
  dnobs_r = dnobs_qr;
  
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_bp = NaN(size(dp_disagg));
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_disagg_adj_bp(:,i) = xbpadj;
  end;
  
  dp_aggs_adj_bp = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_agg));
  dp_aggs_adj = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  for i = 1:size(dp_aggs,2);
      xadj = outlier_adj(dp_aggs(:,i),bw,outlier_mult,repl_value);
      dp_aggs_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_aggs_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_aggs_adj_bp(:,i) = xbpadj;
  end;
 
 % Real Variables
  yvar = 0.5*log(gdp)+0.5*log(gdi); labelvec_r = {'GDO'};
  yvar = [yvar capu]; labelvec_r = [labelvec_r;'Capacity utilization'];
  yvar = [yvar log(emp_est)]; labelvec_r = [labelvec_r;'Employment (establishments)'];
  yvar = [yvar emp_pop]; labelvec_r = [labelvec_r;'Employment/Population'];
  yvar = [yvar -urate]; labelvec_r = [labelvec_r;'Unemployment rate (inverted)'];
  yvar = [yvar -urate_st]; labelvec_r = [labelvec_r;'Short term unemployment rate (inverted)'];

  % Lags used in PC
  pc_lag = [2 2 0 0 0 2]';

  % List variables used and lags
  fprintf('Using real variables: \n');
  for i = 1:size(yvar,2);
        str = char(labelvec_r(i));
        str = [str '  lags =' num2str(pc_lag(i)) '\n'];
        fprintf(str);
  end;

  yvar_bp = NaN(size(yvar));
  for i = 1:size(yvar,2);
      [xbpadj tmp] = bpassar(yvar(:,i),updc,lpdc,n,nar,tcode,seflag);
      yvar_bp(:,i) = xbpadj;
  end;
  
  % Form variable matrix for PC;
  ypc_data = NaN(size(yvar_bp));
  jj = 1;
  for i = 1:size(yvar,2);
        ypc_data(:,jj) = lag(yvar_bp(:,i),pc_lag(i));
        jj = jj+1;
  end;

 first_date = [1984 1];
 last_date = [2018 1];
 ismpl_r = smpl(calvec_r,first_date,last_date,4);
 ismpl_p = smpl(calvec_p,first_date,last_date,4);
 calvec = calvec_r(ismpl_r==1);
 n_t = size(calvec,1);

 % Construct PC
 ypc_data = ypc_data(ismpl_r==1,:);
 ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
 [pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
 ybp_pc = pc_score(:,1);
 ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);

  % Inflations measures over sample period
  dp_aggs_dif = dp_aggs_adj_dif(ismpl_p==1,:);
  dp_disagg_dif = dp_disagg_adj_dif(ismpl_p==1,:);
  dp_aggs_bp = dp_aggs_adj_bp(ismpl_p==1,:);
  dp_disagg_bp = dp_disagg_adj_bp(ismpl_p==1,:);
  dp_aggs = dp_aggs_adj(ismpl_p==1,:);
  dp_disagg = dp_disagg_adj(ismpl_p==1,:);
  
  % Standardize Sector Inflation Measures
  dp_disagg_dif_std = (dp_disagg_dif-mean(dp_disagg_dif))./std(dp_disagg_dif);
  dp_disagg_bp_std = (dp_disagg_bp-mean(dp_disagg_bp))./std(dp_disagg_bp);
  
  
  % Plot Level of Disaggregate Inflation
  pan = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';'n';'o';'p';'q';'r';'s';'t'};
  ax_fs = 15;
  figure;
  for i = 1:17;
      ss = char(labelvec_disagg(i));
      subplot(6,3,i);
      plot(calvec,ybp_pc,'-- b','LineWidth',2);
      hold on;
        plot(calvec,dp_disagg_dif_std(:,i),'- k','LineWidth',3);
      hold off;
      if i == 1
         legend('Cyclical activity index','Inflation component','Location','north');
      end
      xlim([1980 2020]);
      title(ss);
      ax = gca;
      ax.FontSize = ax_fs;
      % Save Values to CSV File
      ss1 = char(pan(i));
      outfile_name = [outdir 'Chart_3_4_' ss1 '.csv'];
      fileID = fopen(outfile_name,'w');
      fprintf(fileID,['date,Cyclical activity index,' ss '\n']);
      tmp = [calvec ybp_pc dp_disagg_dif_std(:,i)];
      prtmat_comma(tmp,fileID,'%12.4f','\n');
      
  end;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_3_4']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
  path(p);  % Reset path
  