% Estimate CSI Weights using EA data (differences) and Activity Index

clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(63761);

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
  
  
  
  % -- Suffix for Saving Results
  save_suffix = '_difference';
  
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  ea_real_data_calendar;
  calvec = calvec_r; 
  % .... Step 1 .. compute Real Index
  
  % Construct Some Real Indicators
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  [capu_bp tmp] = bpassar(capu,updc,lpdc,n,nar,tcode,seflag);
  [ip_bp tmp] = bpassar(log(indpro),updc,lpdc,n,nar,tcode,seflag);
  [urate_bp tmp] = bpassar(urate,updc,lpdc,n,nar,tcode,seflag);
  x = [capu_bp -urate_bp ip_bp];
  x = (x - mean(packr(x)))./std(packr(x));
  xpc = mean(x,2);
  xpc = (xpc - mean(packr(xpc)))/std(packr(xpc));   % Real Activity Index
  
  % Step 2: Read in Inflation Data and Transform
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_calendar_inflation;
  % Data Series Used
  dp_aggs = [dp_all dp_xfeat dp_xepf];
  namevec_aggs = {'dp_all';'dp_xfeat';'dp_xeuf'};
  % Select dissagregated Series to Use
 % ii = [1 1 1 1 1 0 0 1 1 1 1 1 1 1];   % Housing
 % ii = [1 1 1 0 1 0 1 1 1 1 1 1 1 1];   % Housing Rentals
  ii = [1 1 1 0 0 1 1 1 1 1 1 1 1 1];   % Housing X Energy
  dp_disagg = dp_disagg(:,ii==1);
  namevec_disagg = namevec_disagg(ii'==1);
  labelvec_disagg = labelvec_disagg(ii'==1);
  n_p = size(dp_disagg,2);
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:n_p;
      %xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      xadj = dp_disagg(:,i);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
  end;
  
  % Aggregate Series
  tmp = ma(dp_all,4);
  dp_all_dif = dif(tmp,4);
  tmp = ma(dp_xfeat,4);
  dp_xfeat_dif = dif(tmp,4);
  tmp = ma(dp_xepf,4);
  dp_xepf_dif = dif(tmp,4);
  
  % Set up calendars
  first_date = [1998 1];
  last_date = [2018 1];
  ismpl_p = smpl(calvec_p,first_date,last_date,4);
  ismpl_r = smpl(calvec_r,first_date,last_date,4);
  calvec = calvec_r(ismpl_r==1);
  n_t = size(calvec,1);
  
  
  % ------ Compute bencmark CSI ---- 
  % Carry out analyis using fixed lags of xpc;
  nit = 500;
  options = optimset('Display','off');
  nlag = 0;
  w_logit = NaN(n_p,1);
  dp = dp_disagg_adj_dif(ismpl_p==1,:);
  dp = dp-mean(dp);
  ypc = xpc(ismpl_r==1);
  ypc = (ypc-mean(ypc))/std(ypc);
  fdata.y = ypc;
  fdata.z = dp;
  theta_max = zeros(n_p-1,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max;   
  tic;
  for i = 1:nit;
           if floor(i/2) == ceil(i/2);
               theta = theta + .02*randn(n_p-1,1);
           else;
               theta = 0.5*randn(n_p-1,1);
           end;
           theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
           tmp = myfun4(theta,fdata);
           if tmp < fmin;
               theta_max = theta;
               fmin = tmp;
               theta_save = [theta_max theta_save];
               fmin_save = [fmin;fmin_save];
           end;
    end;
    toc;
    w_logit(:,1) = logit_wght(theta_save(:,1));
    R2_csi=-fmin(1);
    csi_dif = dp_disagg_adj_dif*w_logit;
    csi_lev = dp_disagg_adj*w_logit;
    csi_lev_ma4 = ma(csi_lev,4);
    dp_xfeat_ma4 = ma(dp_xfeat,4);
    dp_all_ma4 = ma(dp_all,4);
    dp_xepf_ma4 = ma(dp_xepf,4);
    csi_dif_benchmark = csi_dif;
    csi_lev_benchmark = csi_lev;
    csi_lev_ma4_benchmark = csi_lev_ma4;
    w_logit_benchmark = w_logit;
    R2_csi_benchmark=R2_csi;
    
    
  % Carry out analyis using average of gdp_gap and unemployment rate cap;
  nit = 500;
  options = optimset('Display','off');
  nlag = 0;
  w_logit = NaN(n_p,1);
  dp = dp_disagg_adj_dif(ismpl_p==1,:);
  dp = dp-mean(dp);
  y1 = ugap_ec;
  y2 = gdpgap_imf;
  x = [y2 -y1];
  x = x(ismpl_r==1,:);
  x = (x - mean(x))./std(x);
  ypc_gap = mean(x,2);
  fdata.y = ypc_gap;
  fdata.z = dp;
  theta_max = zeros(n_p-1,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max;   
  tic;
  for i = 1:nit;
           if floor(i/2) == ceil(i/2);
               theta = theta + .02*randn(n_p-1,1);
           else;
               theta = 0.5*randn(n_p-1,1);
           end;
           theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
           tmp = myfun4(theta,fdata);
           if tmp < fmin;
               theta_max = theta;
               fmin = tmp;
               theta_save = [theta_max theta_save];
               fmin_save = [fmin;fmin_save];
           end;
    end;
    toc;
    w_logit(:,1) = logit_wght(theta_save(:,1));
    R2_csi = -fmin(1);
    csi_dif = dp_disagg_adj_dif*w_logit;
    csi_lev = dp_disagg_adj*w_logit;
    csi_lev_ma4 = ma(csi_lev,4);
    csi_dif_gappc = csi_dif;
    csi_lev_gappc = csi_lev;
    csi_lev_ma4_gappc = csi_lev_ma4;
    w_logit_pcgap = w_logit;
    R2_csi_pcgap = R2_csi;
    
  %Plot CSI and Other Inflation Measures 
  ax_fs = 25;
  plot(calvec,csi_lev_ma4_benchmark(ismpl_p==1),'- k','LineWidth',4);
  hold on;
   plot(calvec,csi_lev_ma4_gappc(ismpl_p==1),': r','LineWidth',4);
   plot(calvec,dp_xepf_ma4(ismpl_p==1),'-- b','LineWidth',2);
  hold off;
  legend('CSI','CSI (gap)','HICP-XEUF');
  xlim([1995 2020]);
  ax = gca;
  ax.FontSize = ax_fs;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_4_5a']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
  % Save Values to CSV File
  outfile_name = [outdir 'Chart_4_5a.csv'];
  fileID = fopen(outfile_name,'w');
  fprintf(fileID,['date,CSI,CSI (gap),HICP-XEUF \n']);
  tmp1 = csi_lev_ma4_benchmark(ismpl_p==1);
  tmp2 = csi_lev_ma4_gappc(ismpl_p==1);
  tmp3 = dp_xepf_ma4(ismpl_p==1);
  tmp = [calvec tmp1 tmp2 tmp3];
  prtmat_comma(tmp,fileID,'%12.4f','\n');
  
  % Compute CSI using 3 real activity series;      
  y1 = ugap_ec;
  y2 = gdpgap_imf;
  x = [y2 -y1];
  x = x(ismpl_r==1,:);
  x = (x - mean(x))./std(x);
  fdata.qvar = [ypc x];
  fdata.pvar = dp;
  theta_max = zeros(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max; 
  tic;
  for i = 1:nit;
           if floor(i/2) == ceil(i/2);
               theta = theta + .02*randn(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
           else;
               theta = 0.5*randn(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
           end;
           theta = fminsearch(@(x) myfun1(x,fdata),theta,options); 
           tmp = myfun1(theta,fdata);
           if tmp < fmin;
               theta_max = theta;
               fmin = tmp;
               theta_save = [theta_max theta_save];
               fmin_save = [fmin;fmin_save];
           end;
    end;
    toc;
    n_q = size(fdata.qvar,2);
    n_p = size(fdata.pvar,2);
    xp = theta_save(1:n_p-1,1);
    xq = theta_save(n_p:end,1);
    wp_logit = logit_wght(xp);
    wq_logit = logit_wght(xq);
    R2_csi=-fmin(1);
    csi_dif = dp_disagg_adj_dif*wp_logit;
    csi_lev = dp_disagg_adj*wp_logit;
    csi_lev_ma4 = ma(csi_lev,4);
    csi_dif_3act = csi_dif;
    csi_lev_3act = csi_lev;
    csi_lev_ma4_3act = csi_lev_ma4;
    R2_csi_3act = R2_csi;
    
    
  %Plot CSI and Other Inflation Measures 
  ax_fs = 25;
  plot(calvec,csi_lev_ma4_benchmark(ismpl_p==1),'- k','LineWidth',4);
  hold on;
   plot(calvec,csi_lev_ma4_3act(ismpl_p==1),': r','LineWidth',4);
   plot(calvec,dp_xepf_ma4(ismpl_p==1),'-- b','LineWidth',2);
  hold off;
  legend('CSI','CSI (gaps and CAI)','HICP-XEUF');
  xlim([1995 2020]);
  ax = gca;
  ax.FontSize = ax_fs;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_4_5b']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
  % Save Values to CSV File
  outfile_name = [outdir 'Chart_4_5b.csv'];
  fileID = fopen(outfile_name,'w');
  fprintf(fileID,['date,CSI,CSI (gaps and CAI),HICP-XEUF \n']);
  tmp1 = csi_lev_ma4_benchmark(ismpl_p==1);
  tmp2 = csi_lev_ma4_3act(ismpl_p==1);
  tmp3 = dp_xepf_ma4(ismpl_p==1);
  tmp = [calvec tmp1 tmp2 tmp3];
  prtmat_comma(tmp,fileID,'%12.4f','\n');
    
    
    
    % Save Results
    outfile_name = [outdir 'SomeNumbersFromChart4_5Analysis.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,'Summary for EA -- sensitivity \n');
    fprintf(fileID,'Sample Period: %6.2f to %6.2f \n\n',[calvec(1) calvec(end)]);
    fprintf(fileID,'Sector,Weight Benchmark, Weight GapPC, Weight Activity \n');
    for i = 1:n_p;
        aa = char(labelvec_disagg(i));
        tmp = [aa ','];
        fprintf(fileID,tmp);
        tmp = [w_logit_benchmark(i) w_logit_pcgap(i) wp_logit(i)];
        prtmat_comma(tmp,fileID,'%7.5f','\n');
    end;
    fprintf(fileID,'\n R2,%5.3f,%5.3f,%5.3f \n',[R2_csi_benchmark R2_csi_pcgap R2_csi_3act]);
    fprintf(fileID,'\n Correlation,%5.3f,%5.3f,%5.3f \n \n',[sqrt(R2_csi_benchmark) sqrt(R2_csi_pcgap) sqrt(R2_csi_3act)]);
    fprintf(fileID,'\n \n weights on Q variables in 3ACT Model \n');
    prtmat_comma(wq_logit',fileID,'%7.5f','\n');
    
    
       
path(p);  % Reset path
  
  
