% Estimate Weights .. one real series at a time
% Use Differences of Inflation as target


clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(63761);

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
  
  % -- Suffix for Saving Results
  save_suffix = '_difference';
  
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  ea_real_data_calendar;
  calvec = calvec_r;
  
  % Construct Some Real Indicators
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  % Iteration Parameters
  nit = 500;
  options = optimset('Display','off');
 
  [capu_bp tmp] = bpassar(capu,updc,lpdc,n,nar,tcode,seflag);
  [ip_bp tmp] = bpassar(log(indpro),updc,lpdc,n,nar,tcode,seflag);
  [urate_bp tmp] = bpassar(urate,updc,lpdc,n,nar,tcode,seflag);
  x = [capu_bp -urate_bp ip_bp];
  x = (x - mean(packr(x)))./std(packr(x));
  xpc = mean(x,2);
  xpc = (xpc - mean(packr(xpc)))/std(packr(xpc));
  
  % Figure 4.2
  % Select Series
  first_date = [1995 1];
  last_date = [2018 1];
  ismpl = smpl(calvec,first_date,last_date,4);
  calvec = calvec(ismpl==1);
  
  gdpgap_imf = gdpgap_imf(ismpl==1);
  ugap_ec = ugap_ec(ismpl==1);
  capu_bp = capu_bp(ismpl==1);
  ip_bp = ip_bp(ismpl==1);
  urate_bp = urate_bp(ismpl==1);
  xpc = xpc(ismpl==1);
  
  % Standardize
  m = mean(gdpgap_imf);
  s = std(gdpgap_imf);
  ugap_ec_std = -s*(ugap_ec-mean(ugap_ec))/std(ugap_ec)+m;
  capu_bp_std = s*(capu_bp-mean(capu_bp))/std(capu_bp)+m;
  ip_bp_std = s*(ip_bp-mean(ip_bp))/std(ip_bp)+m;
  urate_bp_std = -s*(urate_bp-mean(urate_bp))/std(urate_bp)+m;
  xpc_std = s*(xpc-mean(xpc))/std(xpc)+m;
  
  ax_fs = 15;
  plot(calvec,gdpgap_imf,': k','LineWidth',2);
  hold on;
    plot(calvec,ugap_ec_std,': b','LineWidth',2);
    plot(calvec,urate_bp_std,'-- b','LineWidth',2);
    plot(calvec,ip_bp_std,'-- k','LineWidth',2);
    plot(calvec,capu_bp_std,'-- r','LineWidth',2);
    plot(calvec,xpc_std,'- k','LineWidth',3);
  hold off;
  legend('Output gap (IMF)','Unemployment rate gap (EC, inverted)','Unemployment rate (bandpass, inverted)','Industrial production (bandpass)','Capacity utilization (bandpass)','Cyclical activity index (bandpass)');
  ax = gca;
  ax.FontSize = ax_fs;
  xlim([1990 2020]);
  
  
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_4_2']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
    
  % Save Values to CSV File
  outfile_name = [outdir 'Chart_4_2.csv'];
  fileID = fopen(outfile_name,'w');
  fprintf(fileID,['date,Output gap (IMF),Unemployment rate gap (EC inverted),Unemployment rate (bandpass inverted),Industrial production (bandpass),Capacity utilization (bandpass),Cyclical activity index (bandpass) \n']);
  tmp = [calvec gdpgap_imf ugap_ec_std urate_bp_std ip_bp_std capu_bp_std xpc_std];
  prtmat_comma(tmp,fileID,'%12.4f','\n');
  
      
path(p);  % Reset path
  
  

  
  