% Plot Sector Inflation


clear all;
small = 1.0e-10;
big = 1.0e+6;                  
%rng(63761);
 
% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);

    
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_calendar_inflation;
  % Data Series Used
  dp_aggs = [dp_all dp_xfeat dp_xepf];
  namevec_aggs = {'dp_all';'dp_xfeat';'dp_xepf'};
  % Select dissagregated Series to Use
 % ii = [1 1 1 1 1 0 0 1 1 1 1 1 1 1];   % Housing
 % ii = [1 1 1 0 1 0 1 1 1 1 1 1 1 1];   % Housing Rentals
  ii = [1 1 1 0 0 1 1 1 1 1 1 1 1 1];   % Housing X Energy
  dp_disagg = dp_disagg(:,ii==1);
  namevec_disagg = namevec_disagg(ii'==1);
  labelvec_disagg = labelvec_disagg(ii'==1);
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      %xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      xadj = dp_disagg(:,i);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
  end;
  
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  dp_aggs_adj = NaN(size(dp_aggs));
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  for i = 1:size(dp_aggs,2);
      %xadj = outlier_adj(dp_aggs(:,i),bw,outlier_mult,repl_value);
      xadj = dp_aggs(:,i);
      dp_aggs_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_aggs_adj_dif(:,i) = dif(tmp,4);
  end;
 
  first_date = [1996 1];
  last_date = [2018 1];
  ismpl = smpl(calvec_p,first_date,last_date,4);
  calvec = calvec_p(ismpl==1);
 
  % Inflations measures over sample period
  dp_aggs_dif = dp_aggs_adj_dif(ismpl==1,:);
  dp_disagg_dif = dp_disagg_adj_dif(ismpl==1,:);
  dp_aggs = dp_aggs_adj(ismpl==1,:);
  dp_disagg = dp_disagg_adj(ismpl==1,:);
  
  % Compute autocovariances of levels and 4q differences 
  lvec = [namevec_aggs;labelvec_disagg];
  dp = [dp_aggs dp_disagg];
  dp_dif = [dp_aggs_dif dp_disagg_dif];
  nac = 12;
  ac_lev = NaN(nac,1);
  ac_dif = NaN(nac,1);
  outfile_name = [outdir 'HICPAutoCors.csv'];
  fileID = fopen(outfile_name,'w');
  ii = (1:1:nac);
  for i = 1:size(dp,2);
      x = dp(:,i);
      y = dp_dif(:,i);
      ac_lev = acor(x,nac);
      ac_dif = acor(y,nac);
      ss = char(lvec(i));
      fprintf(fileID,[ss '\n']);
      prtmat_comma(ii,fileID,'%3i','\n');
      prtmat_comma(ac_lev',fileID,'%5.2f','\n');
      prtmat_comma(ac_dif',fileID,'%5.2f','\n');
  end;
  
  
  % Plot Level of Disaggregate Inflation
  pan = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';'n';'o';'p';'q';'r';'s';'t'};
  ax_fs = 15;
  jj = 1;
  for i = 1:size(dp_disagg,2);
      ss = char(labelvec_disagg(i));
  
      subplot(4,3,i);
      plot(calvec,ma(dp_aggs(:,3),4),'-- b','LineWidth',2);
      hold on;
        plot(calvec,ma(dp_disagg(:,i),4),'- k','LineWidth',3);
      hold off;
      if jj == 1
         legend('HICP-XEUF','Component');
      end
      xlim([1995 2020]);
      title(ss);
      ax = gca;
      ax.FontSize = ax_fs;
      jj = jj+1;
      
      % Save Values to CSV File
      ss1 = char(pan(i));
      outfile_name = [outdir 'Chart_4_1_' ss1 '.csv'];
      fileID = fopen(outfile_name,'w');
      fprintf(fileID,['date,HICP-XEUF,' ss '\n']);
      tmp1 = ma(dp_aggs(:,3),4);
      tmp2 = ma(dp_disagg(:,i),4);
      tmp = [calvec tmp1 tmp2];
      prtmat_comma(tmp,fileID,'%12.4f','\n');
      
      
  end;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_4_1']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
     
path(p);  % Reset path
    

  