% Plot Sector Inflation


clear all;
small = 1.0e-10;
big = 1.0e+6;                  
%rng(63761);
 
% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
    
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  hicp_data_calendar_inflation;
  % Data Series Used
  dp_aggs = [dp_all dp_xfeat dp_xepf];
  namevec_aggs = {'dp_all';'dp_xfeat';'dp_xepf'};
  % Select dissagregated Series to Use
 % ii = [1 1 1 1 1 0 0 1 1 1 1 1 1 1];   % Housing
 % ii = [1 1 1 0 1 0 1 1 1 1 1 1 1 1];   % Housing Rentals
  ii = [1 1 1 0 0 1 1 1 1 1 1 1 1 1];   % Housing X Energy
  dp_disagg = dp_disagg(:,ii==1);
  namevec_disagg = namevec_disagg(ii'==1);
  labelvec_disagg = labelvec_disagg(ii'==1);
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      %xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      xadj = dp_disagg(:,i);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
  end;
  
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  dp_aggs_adj = NaN(size(dp_aggs));
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  for i = 1:size(dp_aggs,2);
      %xadj = outlier_adj(dp_aggs(:,i),bw,outlier_mult,repl_value);
      xadj = dp_aggs(:,i);
      dp_aggs_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_aggs_adj_dif(:,i) = dif(tmp,4);
  end;
 
  first_date = [1996 1];
  last_date = [2018 1];
  ismpl = smpl(calvec_p,first_date,last_date,4);
  
  % Inflations measures over sample period
  dp_aggs_dif = dp_aggs_adj_dif(ismpl==1,:);
  dp_disagg_dif = dp_disagg_adj_dif(ismpl==1,:);
  dp_aggs = dp_aggs_adj(ismpl==1,:);
  dp_disagg = dp_disagg_adj(ismpl==1,:);
  
   % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  ea_real_data_calendar;
  
  % Construct Some Real Indicators
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  [capu_bp tmp] = bpassar(capu,updc,lpdc,n,nar,tcode,seflag);
  [ip_bp tmp] = bpassar(log(indpro),updc,lpdc,n,nar,tcode,seflag);
  [urate_bp tmp] = bpassar(urate,updc,lpdc,n,nar,tcode,seflag);
  x = [capu_bp -urate_bp ip_bp];
  x = (x - mean(packr(x)))./std(packr(x));
  xpc = mean(x,2);
  xpc = (xpc - mean(packr(xpc)))/std(packr(xpc));
  first_date = [1996 1];
  last_date = [2018 1];
  ismpl = smpl(calvec_r,first_date,last_date,4);
  xpc = xpc(ismpl==1);
  calvec = calvec_r(ismpl==1);
  
  % Plot Differnce of Disaggregate Inflation
  pan = {'a';'b';'c';'d';'e';'f';'g';'h';'i';'j';'k';'l';'m';'n';'o';'p';'q';'r';'s';'t'};
  ax_fs = 15;
  jj = 1;
  for i = 1:size(dp_disagg,2);
      ss = char(labelvec_disagg(i));
  
      subplot(4,3,i);
      plot(calvec,xpc,'- b','LineWidth',1);
      hold on;
        plot(calvec,dp_disagg_dif(:,i),'- k','LineWidth',3);
      hold off;
      if jj == 1
         legend('Cyclical Activity Index','Component Inflation');
      end
      xlim([1995 2020]);
      title(ss);
      ax = gca;
      ax.FontSize = ax_fs;
      jj = jj+1;
      
      % Save Values to CSV File
      ss1 = char(pan(i));
      outfile_name = [outdir 'Chart_4_3_' ss1 '.csv'];
      fileID = fopen(outfile_name,'w');
      fprintf(fileID,['date,Cyclical Activity Index,' ss '\n']);
      tmp = [calvec xpc dp_disagg_dif(:,i)];
      prtmat_comma(tmp,fileID,'%12.4f','\n');
      
  end;
  set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
  fig_str = ['Chart_4_3']; str_fig = [figdir fig_str];
  saveas(gcf,[str_fig]);
  saveas(gcf,[str_fig '.png']);
  close(gcf);
  
       
path(p);  % Reset path

  