% Compute Price Indices and write to file
% 6/06/2018, mww

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);

% ----------- Features of Data Set ---------
ns = 26;
miss_code = 1.0e+32;

% ----------- Sample Period, Calendars and so forth
[dnobs_m,calvec_m,calds_m] = calendar_make([1959 1],[2018 3],12);
[dnobs_q,calvec_q,calds_q] = calendar_make([1959 1],[2018 1],4);

% --------------- Read In Quarterly Data for Nominal PCE ---------------- 
xlsname = '../data/US/nom_pce_m59.xlsx';
% Read Data 
ndesc=1;
ncodes=1;
sheet='Sheet1';
[namevec,descmat,tcodemat,datevec,datamat] = readxls(xlsname,sheet,ns,dnobs_m,ndesc,ncodes);
tcodemat = tcodemat(1,:);
labelvec=descmat(:,1);
% Convert Namestrings to upper case 
namevec = upper(namevec);
% Eliminate any leading or trailing blanks 
namevec=strtrim(namevec);
labelvec=strtrim(labelvec);
% Replace missing values with NaN
isel = datamat == miss_code;
datamat(isel) = NaN;
pce_nom = datamat;
namevec_nom = namevec;
labelvec_nom = labelvec;
tcodemat_nom = tcodemat;

% ------------- Read in Price Deflators and compute inflation Rates ---------
xlsname = '../data/US/p_pce_m59.xlsx';
% Read Data 
ndesc=2;
ncodes=2;
sheet='Sheet1';
[namevec,descmat,codemat,datevec,datamat] = readxls(xlsname,sheet,ns,dnobs_m,ndesc,ncodes);
tcodemat = codemat(1,:);
ordermat = codemat(2,:)';
labelvec=descmat(:,1);
labelvec_short = descmat(:,2);
% Convert Namestrings to upper case 
namevec = upper(namevec);
% Eliminate any leading or trailing blanks 
namevec=strtrim(namevec);
labelvec=strtrim(labelvec);
labelvec_short=strtrim(labelvec_short);
% Replace missing values with NaN
isel = datamat == miss_code;
datamat(isel) = NaN;
pce_p = datamat;
namevec_p = namevec;
labelvec_p = labelvec;
labelvec_short_p = labelvec_short;
tcodemat_p = tcodemat;

% Compute Total and Values for Included Series
isel = tcodemat_p==2;
p_agg = pce_p(:,isel==1);         % Aggregrate Price Indenx  
isel = tcodemat==2;
nom_agg = pce_nom(:,isel==1);     % Aggregate Nominal PCE
isel = tcodemat_p==5;
p_agg_xfe = pce_p(:,isel==1);     % Aggregrate Price Index XFE  
isel = tcodemat==5;
nom_agg_xfe = pce_nom(:,isel==1);  % Aggregate Nominal PCE XFE

% Compute Energy Services Component of Household Expenditures
isel = tcodemat==3;
nom_eg = pce_nom(:,isel==1);   % Gasoline and other energy goods
isel = tcodemat==4;
nom_hu = pce_nom(:,isel==1);    % Housing and utilities
isel = tcodemat==6;
nom_egs = pce_nom(:,isel==1);   % Energy Goods and Services
isel = tcodemat_p==3;
p_eg = pce_p(:,isel==1);        % Gasoline and other energy goods
isel = tcodemat_p==4;
p_hu = pce_p(:,isel==1);        % Housing and utilities
isel = tcodemat_p==6;
p_egs = pce_p(:,isel==1);       % Energy Goods and Services

nom_hu_es = nom_egs - nom_eg;    % Energy services component of housing and utilities
nom_hu_xes = nom_hu - nom_hu_es; % Housing and Utilities excluding energy services

% Compute some inflation rates
infl_eg = NaN*zeros(dnobs_m,1);
infl_eg(2:end) = log(p_eg(2:end)./p_eg(1:end-1));
infl_egs = NaN*zeros(dnobs_m,1);
infl_egs(2:end) = log(p_egs(2:end)./p_egs(1:end-1));
s_eg_egs = nom_eg./nom_egs;   % Share of energy goods in energy goods and services 
infl_hu_es = (infl_egs - (s_eg_egs.*infl_eg))./(1-s_eg_egs);
p_hu_es = NaN*zeros(dnobs_m,1);
p_hu_es(1) = 100;
for i = 2:dnobs_m;
    p_hu_es(i) = p_hu_es(i-1)*exp(infl_hu_es(i));
end;
infl_hu = NaN*zeros(dnobs_m,1);
infl_hu(2:end) = log(p_hu(2:end)./p_hu(1:end-1));
s_es_hu = nom_hu_es./nom_hu;  % Share of energy in housing and utilities
infl_hu_xes = (infl_hu - (s_es_hu.*infl_hu_es))./(1-s_es_hu);
p_hu_xes = NaN*ones(dnobs_m,1);
p_hu_xes(1) = 100;
for i = 2:dnobs_m;
    p_hu_xes(i) = p_hu_xes(i-1)*exp(infl_hu_xes(i));
end;

% Add New Series to various matrices
pce_nom = [pce_nom nom_hu_es nom_hu_xes];
tcodemat = [tcodemat 1 1];
namevec = [namevec;'nom_hu_es';'nom_hu_xes'];
labelvec = [labelvec;'Energy Services in Housing and Utilities';'Housing and Utilities excl Energy Services'];

pce_p = [pce_p [p_hu_es p_hu_xes]];
tcodemat_p = [tcodemat_p 1 1];
ordermat_p = [ordermat;9.2;9.1];
namevec_p = [namevec_p;'p_hu_es';'p_hu_xes'];
labelvec_p = [labelvec_p;'Gas & electric untilies';'Housing excluding gas & electric utilities'];
labelvec_short_p = [labelvec_short_p;'Gas & electric untilies';'Housing excl. gas & elec. util.'];

isel = (tcodemat_p==1)+(tcodemat_p==3);
p_disagg = pce_p(:,isel==1);
ordervec = ordermat_p(isel'==1);
nom_disagg = pce_nom(:,isel==1);
namevec_disagg = namevec_p(isel==1);
labelvec_disagg = labelvec_p(isel==1);
labelvec_short_disagg = labelvec_short_p(isel==1);
n_incl = size(p_disagg,2);

% ... Data Check ... 
 % Add nominal disaggegrated PCE and compare to aggregate
 tmp = sum(nom_disagg,2);
 tmp_dif = (tmp - nom_agg)./nom_agg; 
 if max(abs(tmp_dif)) > .0001;
     error('nominal dissaggregates do not add to nominal aggregate');
 end;
 
 % Compute Various Inflation Measures for Aggregate and Dissaggregate PCE
 % -- Monthly data
 p_agg_m = p_agg;
 p_agg_xfe_m = p_agg_xfe;
 p_disagg_m = p_disagg;
 nom_disagg_m = nom_disagg;
 nom_agg_m = nom_agg;
 nom_agg_xfe_m = nom_agg_xfe;
 pce_nom_m = pce_nom;
 nom_egs_m = nom_egs;
 p_egs_m = p_egs;

 % --- Monthly Inflation ----
 paar = 1200;
 dp_agg_m = paar*dif(log(p_agg_m),1);
 dp_agg_xfe_m = paar*dif(log(p_agg_xfe_m),1);
 dp_disagg_m = paar*dif(log(p_disagg_m),1);
 
 
 % Compute dp_agg_xe:  aggregate inflation excluding energy
 % -- Monthly
 s_egs_agg_m = nom_egs_m./nom_agg_m;            % Share of energy goods and services in total
 dp_egs_m = 1200*dif(log(p_egs_m),1);           % Energy goods and services Inflation
 dp_agg_xe_m = (dp_agg_m - s_egs_agg_m.*dp_egs_m)./(1-s_egs_agg_m);
 tmp = [0;dp_agg_xe_m(2:end)/1200];
 tmp = cumsum(tmp);
 p_agg_xe_m = 100*exp(tmp);
 
 % -- Select Data to Write to Spreadsheet
 ss_data = [calds_m p_agg_m p_agg_xfe_m p_agg_xe_m p_disagg_m];
 ss_r1 = ['Year';'Month';'P_agg';'P_agg_xfe';'P_agg_xe';namevec_disagg];
 ss_r2 = [' ';' ';'P Aggregate';'P-XFE Aggregate';'P-XE Aggregate';labelvec_short_disagg];
 ss_r3 = [' ';' ';'Aggregate Price Index';'Aggregate-XFE Price Index';'Aggregate-XE Price Index';labelvec_disagg];
 
 % -- Write SpreadSheet as CSV File --
 filename = [outdir 'Price_Data.csv'];
 fileID = fopen(filename,'w');
 n = size(ss_r1,1);
 for i = 1:n;
     stmp = char(ss_r1(i));
     fprintf(fileID,stmp);
     if i < n;
         fprintf(fileID,',');
     else;
         fprintf(fileID,'\n');
     end;
 end;
 for i = 1:n;
     stmp = char(ss_r2(i));
     fprintf(fileID,stmp);
     if i < n;
         fprintf(fileID,',');
     else;
         fprintf(fileID,'\n');
     end;
 end;
 for i = 1:n;
     stmp = char(ss_r3(i));
     fprintf(fileID,stmp);
     if i < n;
         fprintf(fileID,',');
     else;
         fprintf(fileID,'\n');
     end;
 end;
 for i = 1:size(ss_data,1);
     prtmat_comma(ss_data(i,1:2),fileID,'%i',',');
     prtmat_comma(ss_data(i,3:end),fileID,'%f','\n');
 end;
 
  path(p);  % Reset path
 

