% Estimate CSI .. US data

clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(63761);

 % -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);
  
  
  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_prices;
  % Data Series Used
  dp_agg = dp_agg_q;
  dp_agg_xfe = dp_agg_xfe_q;
  dp_agg_xe = dp_agg_xe_q;
  dp_disagg = dp_disagg_q;
  share_avg = share_avg_q;
  share_avg_xe = share_avg_xe_q;
  share_avg_xfe= share_avg_xfe_q;
  calvec_p = calvec_qp;
  dnobs_p = dnobs_qp;
  calds_p = calds_qp;
  dp_aggs = [dp_agg dp_agg_xfe dp_agg_xe];
  namevec_aggs = {'dp_agg';'dp_agg_xfe';'dp_agg_xe'};
  dp_disagg_zero = [0 0 1 0 0 1 0 0 0 0 0 0 0 0 1 0 1]';  % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
  n_p = size(dp_disagg,2);
  
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_real;
  calvec_r = calvec_qr;
  calds_r = calds_qr;
  dnobs_r = dnobs_qr;
  
  % Iteration Parameters
  nit = 500;
  options = optimset('Display','off');
  
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_bp = NaN(size(dp_disagg));
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_disagg_adj_bp(:,i) = xbpadj;
  end;
  
  dp_aggs_adj_bp = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_agg));
  dp_aggs_adj = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  for i = 1:size(dp_aggs,2);
      xadj = outlier_adj(dp_aggs(:,i),bw,outlier_mult,repl_value);
      dp_aggs_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_aggs_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_aggs_adj_bp(:,i) = xbpadj;
  end;
  
% Real Variables
  yvar = 0.5*log(gdp)+0.5*log(gdi); labelvec_r = {'GDO'};
  yvar = [yvar capu]; labelvec_r = [labelvec_r;'Capacity utilization'];
  yvar = [yvar log(emp_est)]; labelvec_r = [labelvec_r;'Employment (establishments)'];
  yvar = [yvar emp_pop]; labelvec_r = [labelvec_r;'Employment/Population'];
  yvar = [yvar -urate]; labelvec_r = [labelvec_r;'Unemployment rate (inverted)'];
  yvar = [yvar -urate_st]; labelvec_r = [labelvec_r;'Short term unemployment rate (inverted)'];

yvar_bp = NaN(size(yvar));
for i = 1:size(yvar,2);
      [xbpadj tmp] = bpassar(yvar(:,i),updc,lpdc,n,nar,tcode,seflag);
      yvar_bp(:,i) = xbpadj;
end;


% Lags used in PC
pc_lag = [2 2 0 0 0 2]';

% List variables used and lags
fprintf('Using real variables: \n');
for i = 1:size(yvar,2);
        str = char(labelvec_r(i));
        str = [str '  lags =' num2str(pc_lag(i)) '\n'];
        fprintf(str);
end;

% Form variable matrix for PC;
ypc_data = NaN(size(yvar_bp));
jj = 1;
for i = 1:size(yvar,2);
        ypc_data(:,jj) = lag(yvar_bp(:,i),pc_lag(i));
        jj = jj+1;
end;

first_date = [1984 1];
last_date = [2018 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec = calvec_r(ismpl_r==1);
n_t = size(calvec,1);

% Construct PC
ypc_data = ypc_data(ismpl_r==1,:);
ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
[pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
ybp_pc = pc_score(:,1);
ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);

% Step 1:  Regress ybp_pc onto inflation measures .. compute R2
y = ybp_pc;
x = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);
x = x - mean(x);
fdata.z = x;
fdata.y = y;
theta_max = zeros(size(x,2)-1,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max;   
tic;
for i = 1:nit;
  if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
  else;
      theta = 0.5*randn(size(x,2)-1,1);
  end;
  theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
  tmp = myfun4(theta,fdata);
  if tmp < fmin;
    theta_max = theta;
    fmin = tmp;
    theta_save = [theta_max theta_save];
    fmin_save = [fmin;fmin_save];
  end;
end;
b_logit = logit_wght(theta_save(:,1));
w_logit = zeros(n_p,1);
w_logit(dp_non_zero==1) = b_logit;

% Construct CSI .. level, difference, BP
csi_level = dp_disagg_adj(ismpl_p==1,:)*w_logit;
csi_dif = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit;
csi_bp = dp_disagg_adj_bp(ismpl_p==1,:)*w_logit;

% Compute correlation of series with cyclical activity indicator
r_bp = NaN(n_p,1);
r_dif = NaN(n_p,1);
for i = 1:n_p;
    x = dp_disagg_adj_bp(ismpl_p==1,i);
    y = ybp_pc;
    r_bp(i) = cor_compute(x,y);
    x = dp_disagg_adj_dif(ismpl_p==1,i);
    y = ybp_pc;
    r_dif(i) = cor_compute(x,y);
end;

% Save Results to Table
% Save Results
    outfile_name = [outdir 'Table_3_2.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,'Summary for US \n');
    fprintf(fileID,'Sample Period: %6.2f to %6.2f \n\n',[calvec(1) calvec(end)]);
    fprintf(fileID,'Sector,BP Cor, Dif Cor,CSI Weight \n');
    for i = 1:n_p;
        aa = char(labelvec_disagg(i));
        tmp = [aa ','];
        fprintf(fileID,tmp);
        tmp = [r_bp(i) r_dif(i) w_logit(i)];
        prtmat_comma(tmp,fileID,'%7.5f','\n');
    end;
    
    r_csi_bp = cor_compute(csi_bp,ybp_pc);
    r_csi_dif = cor_compute(csi_dif,ybp_pc);
    fprintf(fileID,'\n CSI,%7.5f, %7.5f \n',[r_csi_bp r_csi_dif]);
    
    % Compute Results for Figure 3.5
    first_date = [1960 1];
    last_date = [2018 1];
    ismpl_p = smpl(calvec_p,first_date,last_date,4);
    ismpl_r = smpl(calvec_r,first_date,last_date,4);
    csi_dif = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit;
    unemp_bp = yvar_bp(ismpl_r==1,5);
    calvec = calvec_p(ismpl_p==1);
    
    ax_fs = 25;
    plot(calvec,100*unemp_bp,'-- b','LineWidth',3);
    hold on;
     plot(calvec,csi_dif,'- k','LineWidth',3);
    hold off;
    legend('Unemployment Rate (cyclical component, inverted)','CSI (4Q change of 4Q inflation)','Location','north');
    xlim([1960 2020]);
    ax = gca;
    ax.FontSize = ax_fs;
    h=vline(1984,'- k','');
    set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
    fig_str = ['Chart_3_5']; str_fig = [figdir fig_str];
    saveas(gcf,[str_fig]);
    saveas(gcf,[str_fig '.png']);
    close(gcf);
    
    % Save Values to CSV File
    outfile_name = [outdir 'Chart_3_5.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,['date,Unemployment Rate (cyclical component inverted),CSI (4Q change of 4Q inflation) \n']);
    tmp = [calvec 100*unemp_bp csi_dif];
    prtmat_comma(tmp,fileID,'%12.4f','\n');
    
    % Figure 3.6
    first_date = [1960 1];
    last_date = [2018 1];
    ismpl_p = smpl(calvec_p,first_date,last_date,4);
    csi_level = dp_disagg_adj*w_logit;
    dp_pceall = dp_aggs(:,1);
    dp_pcexfe = dp_aggs(:,2);
    csi_level_ma4 = ma(csi_level,4);
    dp_pceall_ma4 = ma(dp_pceall,4);
    dp_pcexfe_ma4 = ma(dp_pcexfe,4);
    calvec = calvec_p(ismpl_p==1);
    
    ax_fs = 25;
    plot(calvec,csi_level_ma4(ismpl_p==1),'- k','LineWidth',3);
    hold on;
     plot(calvec,dp_pceall_ma4(ismpl_p==1),'-- b','LineWidth',3);
     plot(calvec,dp_pcexfe_ma4(ismpl_p==1),': r','LineWidth',3);
    hold off;
    legend('CSI','PCE-ALL','PCE-XFE','Location','north');
    legend('AutoUpdate','off');
    xlim([1960 2020]);
    ylim([-2 12]);
    ax = gca;
    ax.FontSize = ax_fs;
    x_lim = ax.XLim;
    y_lim = ax.YLim;
    recession_bars(x_lim,y_lim);
    set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
    fig_str = ['Chart_3_6']; str_fig = [figdir fig_str];
    saveas(gcf,[str_fig]);
    saveas(gcf,[str_fig '.png']);
    close(gcf);
    
    % Save Values to CSV File
    outfile_name = [outdir 'Chart_3_6.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,['date,CSI,PCE-ALL,PCE-XFE \n']);
    tmp = [calvec csi_level_ma4(ismpl_p==1) dp_pceall_ma4(ismpl_p==1) dp_pcexfe_ma4(ismpl_p==1)];
    prtmat_comma(tmp,fileID,'%12.4f','\n');
    
    % Compute Correlations over different subsamples:
    first_date = [1960 1];
    last_date = [1983 4];
    ismpl_p1 = smpl(calvec_p,first_date,last_date,4);
    ismpl_r1 = smpl(calvec_r,first_date,last_date,4);
    x = dp_disagg_adj_dif(ismpl_p1==1,:)*w_logit;
    y = yvar_bp(ismpl_r1==1,5);
    tmp = packr([x y]);
    x = tmp(:,1);
    y = tmp(:,2);
    r1 = cor_compute(x,y);
    first_date = [1984 1];
    last_date = [2018 1];
    ismpl_p2 = smpl(calvec_p,first_date,last_date,4);
    ismpl_r2 = smpl(calvec_r,first_date,last_date,4);
    x = dp_disagg_adj_dif(ismpl_p2==1,:)*w_logit;
    y = yvar_bp(ismpl_r2==1,5);
    r2 = cor_compute(x,y);
    fprintf('Correlations from Figure 3.5 over both sample periods: %5.2f %5.2f \n',[r1 r2]);
    first_date = [1960 1];
    last_date = [2018 1];
    ismpl_p3 = smpl(calvec_p,first_date,last_date,4);
    ismpl_r3 = smpl(calvec_r,first_date,last_date,4);
    x = dp_disagg_adj_dif*w_logit;
    x = [x x.*ismpl_p1];
    x = x(ismpl_p3 == 1,:);
    y = yvar_bp(ismpl_r3==1,5);
    tmp = packr([y x]);
    y = tmp(:,1);
    x = tmp(:,2:end);
    x = [ones(size(x,1),1) x];
    [betahat,vbeta,se_beta,ser,rbarsq] = hac_ols(y,x,8,1);
    fprintf('Chow t-test for change in coefficient: %5.2f \n',betahat(end)/se_beta(end));
    
    % Table 3.3
    R_Mat = zeros(3,3);
    ismpl_p_1 = smpl(calvec_p,[1960 4],[1983 4],4);
    ismpl_p_2 = smpl(calvec_p,[1984 1],[1999 4],4);
    ismpl_p_3 = smpl(calvec_p,[2000 1],[2018 1],4);
    ismpl_r_1 = smpl(calvec_r,[1960 4],[1983 4],4);
    ismpl_r_2 = smpl(calvec_r,[1984 1],[1999 4],4);
    ismpl_r_3 = smpl(calvec_r,[2000 1],[2018 1],4);
    ismpl_p = [ismpl_p_1 ismpl_p_2 ismpl_p_3];
    ismpl_r = [ismpl_r_1 ismpl_r_2 ismpl_r_3];
    dp = [dp_aggs_adj_dif(:,1:2) dp_disagg_adj_dif*w_logit];
    for ip = 1:3;
        for it = 1:3;
            y = yvar_bp(ismpl_r(:,it)==1,5);
            x = dp(ismpl_p(:,it)==1,ip);
            R_Mat(ip,it) = cor_compute(x,y);
        end;
    end;
    
    % Save Results
    outfile_name = [outdir 'Table_3_3.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,'Correlations of 4Qdif of 4Q inflation with BP unemployment Rate \n');
    fprintf(fileID,'Series, 1960:4-1983:4, 1984:1 1999:4, 2001:1-2018:1 \n');
    fprintf(fileID,'PCE-ALL,');
    prtmat_comma(R_Mat(1,:),fileID,'%5.3f','\n');
    fprintf(fileID,'PCE-XFE,');
    prtmat_comma(R_Mat(2,:),fileID,'%5.3f','\n');
    fprintf(fileID,'CSI,');
    prtmat_comma(R_Mat(3,:),fileID,'%5.3f','\n');
    
    % Save Value of CSI to File
    outfile_name = [outdir 'US_CSI.csv'];
    fileID = fopen(outfile_name,'w');
    first_date = [1984 1];
    last_date = [2018 1];
    ismpl_p = smpl(calvec_p,first_date,last_date,4);
    calvec = calvec_p(ismpl_p==1);
    fprintf(fileID,'CSI Values from US_Estimate_CSI.m \n\n');
    fprintf(fileID,'Date,Level,4QMA,4QDif of 4QMA, BP \n');
    % Construct CSI .. level, difference, BP
    csi_level = dp_disagg_adj(ismpl_p==1,:)*w_logit;
    tmp = ma(dp_disagg_adj*w_logit,4);
    csi_level_ma4 = tmp(ismpl_p==1);
    csi_dif = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit;
    csi_bp = dp_disagg_adj_bp(ismpl_p==1,:)*w_logit;
    tmp = [calvec csi_level csi_level_ma4 csi_dif csi_bp];
    prtmat_comma(tmp,fileID,'%5.2f','\n');
    
    % Save Results
    outfile_name = [outdir 'Table_3_3.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,'Correlations of 4Qdif of 4Q inflation with BP unemployment Rate \n');
    fprintf(fileID,'Series, 1960:4-1983:4, 1984:1 1999:4, 2001:1-2018:1 \n');
    fprintf(fileID,'PCE-ALL,');
    prtmat_comma(R_Mat(1,:),fileID,'%5.3f','\n');
    fprintf(fileID,'PCE-XFE,');
    prtmat_comma(R_Mat(2,:),fileID,'%5.3f','\n');
    fprintf(fileID,'CSI,');
    prtmat_comma(R_Mat(3,:),fileID,'%5.3f','\n');
    
    % Save Value of CSI to File
    outfile_name = [outdir 'US_CSI.csv'];
    fileID = fopen(outfile_name,'w');
    first_date = [1984 1];
    last_date = [2018 1];
    ismpl_p = smpl(calvec_p,first_date,last_date,4);
    calvec = calvec_p(ismpl_p==1);
    fprintf(fileID,'CSI Values from US_Estimate_CSI.m \n\n');
    fprintf(fileID,'Date,Level,4QMA,4QDif of 4QMA, BP \n');
    % Construct CSI .. level, difference, BP
    csi_level = dp_disagg_adj(ismpl_p==1,:)*w_logit;
    tmp = ma(dp_disagg_adj*w_logit,4);
    csi_level_ma4 = tmp(ismpl_p==1);
    csi_dif = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit;
    csi_bp = dp_disagg_adj_bp(ismpl_p==1,:)*w_logit;
    tmp = [calvec csi_level csi_level_ma4 csi_dif csi_bp];
    prtmat_comma(tmp,fileID,'%12.4f','\n');
    
    
  path(p);  % Reset path
    
    
    

  
  