% Estimate CSI .. US data

clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(63761);

% -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  procs_dir = '../m_utilities/';            % Directory for necessary matlab_procs and functions
  p = path;   % Update path to access procs_dir                              
  p1=path(procs_dir,p);

  % -- Read in Data --- 
  load_data = 1;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_prices;
  % Data Series Used
  dp_agg = dp_agg_q;
  dp_agg_xfe = dp_agg_xfe_q;
  dp_agg_xe = dp_agg_xe_q;
  dp_disagg = dp_disagg_q;
  share_avg = share_avg_q;
  share_avg_xe = share_avg_xe_q;
  share_avg_xfe= share_avg_xfe_q;
  calvec_p = calvec_qp;
  dnobs_p = dnobs_qp;
  calds_p = calds_qp;
  dp_aggs = [dp_agg dp_agg_xfe dp_agg_xe];
  namevec_aggs = {'dp_agg';'dp_agg_xfe';'dp_agg_xe'};
  dp_disagg_zero = [0 0 1 0 0 1 0 0 0 0 0 0 0 0 1 0 1]';  % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
  n_p = size(dp_disagg,2);
  
  load_data = 0;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_real;
  calvec_r = calvec_qr;
  calds_r = calds_qr;
  dnobs_r = dnobs_qr;
  ugap = 100*urate-u_nr;
  gdpgap = 100*(log(gdp./gdp_pot));
  
  % Iteration Parameters
  nit = 500;
  options = optimset('Display','off');
  
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  % Outlier Parameters
  bw = 33;
  outlier_mult = 4;
  repl_value = 1;
 
  % Construct 4-quarter differences of 4Q inflation
  dp_disagg_adj_bp = NaN(size(dp_disagg));
  dp_disagg_adj_dif = NaN(size(dp_disagg));
  dp_disagg_adj = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      xadj = outlier_adj(dp_disagg(:,i),bw,outlier_mult,repl_value);
      dp_disagg_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_disagg_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_disagg_adj_bp(:,i) = xbpadj;
  end;
  
  dp_aggs_adj_bp = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_agg));
  dp_aggs_adj = NaN(size(dp_agg));
  dp_aggs_adj_dif = NaN(size(dp_aggs));
  for i = 1:size(dp_aggs,2);
      xadj = outlier_adj(dp_aggs(:,i),bw,outlier_mult,repl_value);
      dp_aggs_adj(:,i) = xadj;
      tmp = ma(xadj,4);
      dp_aggs_adj_dif(:,i) = dif(tmp,4);
      [xbpadj tmp] = bpassar(xadj,updc,lpdc,n,nar,tcode,seflag);
      dp_aggs_adj_bp(:,i) = xbpadj;
  end;
  
% Real Variables
  yvar = 0.5*log(gdp)+0.5*log(gdi); labelvec_r = {'GDO'};
  yvar = [yvar capu]; labelvec_r = [labelvec_r;'Capacity utilization'];
  yvar = [yvar log(emp_est)]; labelvec_r = [labelvec_r;'Employment (establishments)'];
  yvar = [yvar emp_pop]; labelvec_r = [labelvec_r;'Employment/Population'];
  yvar = [yvar -urate]; labelvec_r = [labelvec_r;'Unemployment rate (inverted)'];
  yvar = [yvar -urate_st]; labelvec_r = [labelvec_r;'Short term unemployment rate (inverted)'];

yvar_bp = NaN(size(yvar));
for i = 1:size(yvar,2);
      [xbpadj tmp] = bpassar(yvar(:,i),updc,lpdc,n,nar,tcode,seflag);
      yvar_bp(:,i) = xbpadj;
end;


% Lags used in PC
pc_lag = [2 2 0 0 0 2]';

% List variables used and lags
fprintf('Using real variables: \n');
for i = 1:size(yvar,2);
        str = char(labelvec_r(i));
        str = [str '  lags =' num2str(pc_lag(i)) '\n'];
        fprintf(str);
end;

% Form variable matrix for PC;
ypc_data = NaN(size(yvar_bp));
jj = 1;
for i = 1:size(yvar,2);
        ypc_data(:,jj) = lag(yvar_bp(:,i),pc_lag(i));
        jj = jj+1;
end;

first_date = [1984 1];
last_date = [2018 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec = calvec_r(ismpl_r==1);
n_t = size(calvec,1);

% Construct PC
ypc_data_save = ypc_data;
ypc_data = ypc_data_save(ismpl_r==1,:);
ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
[pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
ybp_pc = pc_score(:,1);
ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);

% Step 1:  Regress ybp_pc onto inflation measures .. compute R2
y = ybp_pc;
x = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);
x = x - mean(x);
fdata.z = x;
fdata.y = y;
theta_max = zeros(size(x,2)-1,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max;   
tic;
for i = 1:nit;
  if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
  else;
      theta = 0.5*randn(size(x,2)-1,1);
  end;
  theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
  tmp = myfun4(theta,fdata);
  if tmp < fmin;
    theta_max = theta;
    fmin = tmp;
    theta_save = [theta_max theta_save];
    fmin_save = [fmin;fmin_save];
  end;
end;
b_logit = logit_wght(theta_save(:,1));
w_logit = zeros(n_p,1);
w_logit(dp_non_zero==1) = b_logit;
w_logit_benchmark = w_logit;
r_benchmark = -fmin_save(1);

% Repeat using UGAP as y measure;
tmp = ugap(ismpl_r==1);
ugap_std = (tmp-mean(tmp))/std(tmp);
y = ugap_std;
x = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);
x = x - mean(x);
fdata.z = x;
fdata.y = y;
theta_max = zeros(size(x,2)-1,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max;   
tic;
for i = 1:nit;
  if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
  else;
      theta = 0.5*randn(size(x,2)-1,1);
  end;
  theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
  tmp = myfun4(theta,fdata);
  if tmp < fmin;
    theta_max = theta;
    fmin = tmp;
    theta_save = [theta_max theta_save];
    fmin_save = [fmin;fmin_save];
  end;
end;
b_logit = logit_wght(theta_save(:,1));
w_logit = zeros(n_p,1);
w_logit(dp_non_zero==1) = b_logit;
w_logit_ugap = w_logit;
r_ugap = -fmin_save(1);

% Repeat using BP inflation in place of 4q differences
% Step 1:  Regress ybp_pc onto inflation measures .. compute R2
y = ybp_pc;
x = dp_disagg_adj_bp(ismpl_p==1,dp_non_zero'==1);
x = x - mean(x);
fdata.z = x;
fdata.y = y;
theta_max = zeros(size(x,2)-1,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max;   
tic;
for i = 1:nit;
  if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
  else;
      theta = 0.5*randn(size(x,2)-1,1);
  end;
  theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
  tmp = myfun4(theta,fdata);
  if tmp < fmin;
    theta_max = theta;
    fmin = tmp;
    theta_save = [theta_max theta_save];
    fmin_save = [fmin;fmin_save];
  end;
end;
b_logit = logit_wght(theta_save(:,1));
w_logit = zeros(n_p,1);
w_logit(dp_non_zero==1) = b_logit;
w_logit_bp = w_logit;
r_bp = -fmin_save(1);

% Repeat using all inflation series
% Step 1:  Regress ybp_pc onto inflation measures .. compute R2
y = ybp_pc;
x = dp_disagg_adj_dif(ismpl_p==1,:);
x = x - mean(x);
fdata.z = x;
fdata.y = y;
theta_max = zeros(size(x,2)-1,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max;   
tic;
for i = 1:nit;
  if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
  else;
      theta = 0.5*randn(size(x,2)-1,1);
  end;
  theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
  tmp = myfun4(theta,fdata);
  if tmp < fmin;
    theta_max = theta;
    fmin = tmp;
    theta_save = [theta_max theta_save];
    fmin_save = [fmin;fmin_save];
  end;
end;
b_logit = logit_wght(theta_save(:,1));
w_logit = zeros(n_p,1);
w_logit(:) = b_logit;
w_logit_allseries = w_logit;
r_allseries = -fmin_save(1);

[w_logit_benchmark w_logit_bp w_logit_allseries]
[r_benchmark r_bp r_allseries]

tmp1 = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit_benchmark;
tmp2 = dp_disagg_adj_dif(ismpl_p==1,:)*w_logit_allseries;
r = cor_compute(tmp1,tmp2);

% Compute Rolling estimates
nroll = 60;              % Rolling weights over nroll periods
first_date = [1969 2];   % This is 1984:1 minus 40 quarters
last_date = [2018 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec = calvec_r(ismpl_r==1);
n_t = size(calvec,1);

% Construct PC
ypc_data = ypc_data_save(ismpl_r==1,:);
ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
[pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
ybp_pc = pc_score(:,1);
ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);
dp = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);


% Compute rolling results
nfirst = 1;
nit_1 = 500;
nit_2 = 10;

w_logit_rolling = NaN(n_t,17);

for i_t = nroll:n_t;
  calvec(i_t)
  % OLS Results
    y = ybp_pc(i_t-nroll+1:i_t);
    x = dp(i_t-nroll+1:i_t,:);
    x = x - mean(x);
    fdata.z = x;
    fdata.y = y;  
    if nfirst == 1
       theta = 0.5*randn(size(x,2)-1,1);
       fmin = 0;
       theta_max = zeros(size(x,2)-1,1);
       nit = nit_1;
    else
       theta = theta_save(:,1);
       fmin = myfun4(theta,fdata);
       theta_max = theta;
       nit = nit_2;
    end
    nfirst = 0;
    fmin_save = fmin;
    theta_save = theta_max; 
    for i = 1:nit;
        if i > 1
            theta = 0.5*randn(size(x,2)-1,1);
        end
        theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
        tmp = myfun4(theta,fdata);
        if tmp < fmin
          theta_max = theta;
          fmin = tmp;
          theta_save = [theta_max theta_save];
          fmin_save = [fmin;fmin_save];
        end
    end;
    b_logit = logit_wght(theta_save(:,1));
    w_logit = zeros(n_p,1);
    w_logit(dp_non_zero==1) = b_logit;
    w_logit_rolling(i_t,:) = w_logit';
end;
tmp = NaN(size(dp_disagg_adj_dif));
tmp(ismpl_p==1,:) = w_logit_rolling;
w_logit_rolling = tmp;

% Construct CSI with Different weights
csi_benchmark = dp_disagg_adj_dif*w_logit_benchmark;
csi_bp = dp_disagg_adj_dif*w_logit_bp;
csi_allseries = dp_disagg_adj_dif*w_logit_bp;
csi_ugap = dp_disagg_adj_dif*w_logit_ugap;
csi_rolling = sum(dp_disagg_adj_dif.*w_logit_rolling,2);

% Figure 3.8
first_date = [1984 1];
last_date = [2018 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec = calvec_r(ismpl_r==1);

ax_fs = 25;
plot(calvec,csi_benchmark(ismpl_p==1),'- k','LineWidth',3);
hold on;
 plot(calvec,csi_rolling(ismpl_p==1),'-- b','LineWidth',3);
hold off;
legend('CSI (benchmark weights)','CSI (rolling weights)','Location','south');
xlim([1980 2020]);
ax = gca;
ax.FontSize = ax_fs;
set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
fig_str = ['Chart_3_8a']; str_fig = [figdir fig_str];
saveas(gcf,[str_fig]);
saveas(gcf,[str_fig '.png']);
close(gcf);
% Save Values to CSV File
outfile_name = [outdir 'Chart_3_8_a.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,['date,CSI (benchmark weights),CSI (rolling weights) \n']);
tmp = [calvec csi_benchmark(ismpl_p==1) csi_rolling(ismpl_p==1)];
prtmat_comma(tmp,fileID,'%12.4f','\n');


plot(calvec,csi_benchmark(ismpl_p==1),'- k','LineWidth',3);
hold on;
 plot(calvec,csi_bp(ismpl_p==1),'-- b','LineWidth',3);
hold off;
legend('CSI (benchmark weights)','CSI (band-pass weights)','Location','south');
xlim([1980 2020]);
ax = gca;
ax.FontSize = ax_fs;
set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
fig_str = ['Chart_3_8b']; str_fig = [figdir fig_str];
saveas(gcf,[str_fig]);
saveas(gcf,[str_fig '.png']);
close(gcf);
% Save Values to CSV File
outfile_name = [outdir 'Chart_3_8_b.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,['date,CSI (benchmark weights),CSI (band-pass weights) \n']);
tmp = [calvec csi_benchmark(ismpl_p==1) csi_bp(ismpl_p==1)];
prtmat_comma(tmp,fileID,'%12.4f','\n');


% Figure 3.9
plot(calvec,csi_benchmark(ismpl_p==1),'- k','LineWidth',3);
hold on;
 plot(calvec,csi_allseries(ismpl_p==1),'-- b','LineWidth',3);
hold off;
legend('CSI (benchmark weights)','CSI (17 Sectors)','Location','south');
xlim([1980 2020]);
ax = gca;
ax.FontSize = ax_fs;
set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
fig_str = ['Chart_3_9a']; str_fig = [figdir fig_str];
saveas(gcf,[str_fig]);
saveas(gcf,[str_fig '.png']);
close(gcf);
% Save Values to CSV File
outfile_name = [outdir 'Chart_3_9_a.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,['date,CSI (benchmark weights),CSI (17 Sectors) \n']);
tmp = [calvec csi_benchmark(ismpl_p==1) csi_allseries(ismpl_p==1)];
prtmat_comma(tmp,fileID,'%12.4f','\n');


plot(calvec,csi_benchmark(ismpl_p==1),'- k','LineWidth',3);
hold on;
 plot(calvec,csi_ugap(ismpl_p==1),'-- b','LineWidth',3);
hold off;
legend('CSI (benchmark weights)','CSI (Unemployment gap)','Location','south');
xlim([1980 2020]);
ax = gca;
ax.FontSize = ax_fs;
set(gcf, 'Position', get(0, 'Screensize'));  % Full Screen
fig_str = ['Chart_3_9b']; str_fig = [figdir fig_str];
saveas(gcf,[str_fig]);
saveas(gcf,[str_fig '.png']);
close(gcf);
% Save Values to CSV File
outfile_name = [outdir 'Chart_3_9_b.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,['date,CSI (benchmark weights),CSI (Unemployment gap) \n']);
tmp = [calvec csi_benchmark(ismpl_p==1) csi_ugap(ismpl_p==1)];
prtmat_comma(tmp,fileID,'%12.4f','\n');



% Compute Weights using activity index + gdpgap + (-ugap);
% Compute Weights using activity index + gdpgap + (-ugap);

first_date = [1984 1];
last_date = [2018 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec = calvec_r(ismpl_r==1);
n_t = size(calvec,1);

% Construct PC
ypc_data = ypc_data_save(ismpl_r==1,:);
ypc_data_normalized = (ypc_data-mean(ypc_data))./std(ypc_data);
[pc_coef,pc_score,pc_latent] = pca(ypc_data_normalized);
ybp_pc = pc_score(:,1);
ybp_pc = (ybp_pc - mean(ybp_pc))/std(ybp_pc);
dp = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);

y1 = ybp_pc;;
y2 = gdpgap(ismpl_r==1);
y3 = -ugap(ismpl_r==1);
y = [y1 y2 y3];
y = (y - mean(y))./std(y);
x = dp_disagg_adj_dif(ismpl_p==1,dp_non_zero'==1);
x = x - mean(x);
fdata.qvar = y;
fdata.pvar = x;
theta_max = zeros(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
fmin = 0;
fmin_save = fmin;
theta_save = theta_max; 
tic;
nit=500;
for i = 1:nit;
           if floor(i/2) == ceil(i/2);
               theta = theta + .02*randn(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
           else;
               theta = 0.5*randn(size(fdata.qvar,2)+size(fdata.pvar,2)-2,1);
           end;
           theta = fminsearch(@(x) myfun1(x,fdata),theta,options); 
           tmp = myfun1(theta,fdata);
           if tmp < fmin;
               theta_max = theta;
               fmin = tmp;
               theta_save = [theta_max theta_save];
               fmin_save = [fmin;fmin_save];
           end;
  end;
    toc;
    n_q = size(fdata.qvar,2);
    n_p = size(fdata.pvar,2);
    xp = theta_save(1:n_p-1,1);
    xq = theta_save(n_p:end,1);
    wp_logit = zeros(17,1);
    wp_logit(dp_non_zero==1) = logit_wght(xp);
    wq_logit = logit_wght(xq);
    R2_csi=-fmin(1);
    csi_dif = dp_disagg_adj_dif*wp_logit;
    csi_lev = dp_disagg_adj*wp_logit;
    csi_lev_ma4 = ma(csi_lev,4);
    
    % Save Results
    outfile_name = [outdir 'Table_3_xx_numbers.csv'];
    fileID = fopen(outfile_name,'w');
    fprintf(fileID,'Summary for US -- sensitivity \n');
    fprintf(fileID,'Sample Period: %6.2f to %6.2f \n\n',[calvec(1) calvec(end)]);
    fprintf(fileID,'Sector,Weight Benchmark, Weight Alternative \n');
    for i = 1:n_p;
        aa = char(labelvec_disagg(i));
        tmp = [aa ','];
        fprintf(fileID,tmp);
        tmp = [w_logit_benchmark(i) wp_logit(i)];
        prtmat_comma(tmp,fileID,'%7.5f','\n');
    end;
    fprintf(fileID,'\n R2,%5.3f,%5.3f \n',[r_benchmark R2_csi]);
    fprintf(fileID,'\n \n weights on Q variables in alternative \n');
    prtmat_comma(wq_logit',fileID,'%7.5f','\n');


   
path(p);  % Reset path
    


  