clear
cap log close
set scheme s1color
cap program drop _all
cd "C:\Users\\`c(username)'\Dropbox\SW\Sintra\replication_files\stata_programs"
log using "sintra_sec1-2_replication_us.txt", text replace
set more 1
*************************************************************
* sintra_sec1-2_replication_us.do            JS 6/23/18
*************************************************************
*
*   Read in data and do initial transformations for Sintra paper
*
* Disaggregate price series:
* pce_all:	PCE-All items (inf@ar)
* pce_xfe:	PCE-xfe (inf@ar)
* pce_xe:	PCE-xe (inf@ar)
* dmotrg:	Motor vehicles and parts (inf@ar)
* dfdhrg:	Furnishings & durable hh eqpt (inf@ar)
* dreqrg:	Recreational goods & vehicles (inf@ar)
* dodgrg:	Other durable goods (inf@ar)
* dfxarg:	Food & beverages-off-premises (inf@ar)
* dclorg:	Clothing & footwear (inf@ar)	
* dgoerg:	Gasoline & other energy goods (inf@ar)
* dongrg:	Other nondurable goods (inf@ar)
* dhlcrg:	Health care (inf@ar)
* dtrsrg:	Transportation services (inf@ar)
* drcarg:	Recreation services (inf@ar)
* dfsarg:	Food services & accommodations (inf@ar)
* difsrg:	Financial services & insurance (inf@ar)
* dotsrg:	Other services (inf@ar)
* dnpirg:	NPISH
* p_hu_es:	Gas & electric util (inf@ar)
* p_hu_xe:	Housing ex gas & electric util (inf@ar)
*
************************************************************
* Pull together spreadsheet data for components, convert to quarterly as needed,
* merge with macro_update,
************************************************************
*
clear
import excel "data\Price_Data_US.xlsx", sheet("mprice_tostata") firstrow case(lower)
rename p_agg pce_all
rename p_agg_xfe pce_xfe
rename p_agg_xe pce_xe
rename p_hu_xes p_hu_xe
gen time = m(1959m1)+_n-1
format time %tm
foreach y of varlist pce* d* p_hu* {
 ren `y' `y'm
 gen `y'l = (`y'm + `y'm[_n-1] + `y'm[_n-2])/3
 gen `y' = 400*ln(`y'l/`y'l[_n-3])
 drop `y'm `y'l
}
keep if (month==3)|(month==6)|(month==9)|(month==12)
gen quarter = month/3
drop month
replace time = q(1959q1) + _n-1
format time %tq
tsset time
sort time
label var pce_all "PCE-Aggregate"
label var pce_xfe "P-XFE Aggregate"
label var pce_xe "P-XE Aggregate"
label var dmotrg "Motor vehicles and parts"
label var dfdhrg "Furn & dur household equip"
label var dreqrg "Rec goods & vehicles"
label var dodgrg "Other durable goods"
label var dfxarg "Food & bev off-premises"
label var dclorg "Clothing & footwear"
label var dgoerg "Gasoline & other energy goods"
label var dongrg "Other nondurables goods"
label var dhlcrg "Health care"
label var dtrsrg "Transportation services"
label var drcarg "Recreation services"
label var dfsarg "Food serv & accom"
label var difsrg "Fin services & insurance"
label var dotsrg "Other services"
label var dnpirg "NPISH"
label var p_hu_es "Gas & electric untilies"
label var p_hu_xe "Housing ex gas & elec util"
save data\p_pce_q59.dta, replace
desc
clear
****************************************************
*           Create merged data set 
****************************************************
use data\macro_update_q.dta
merge 1:1 time using data\p_pce_q59
 list time pcepilfe dmotrg
 drop _merge
/*
merge 1:1 time using data\BP_Results
 list time pcepilfe dmotrg
 drop _merge
*/
qui drop quarter year
gen quarter = quarter(dofq(time))
gen year = year(dofq(time))
*
cap drop pce_all pce_xfe
gen pce_all = 400*ln(pcepi/L.pcepi)
gen pce_xfe = 400*ln(pcepilfe/L.pcepilfe)
gen cpi_all = 400*ln(cpiaucsl/L.cpiaucsl)
gen cpi_xfe = 400*ln(cpilfesl/L.cpilfesl)
gen ahe = 400*ln(ahetpi/L.ahetpi)
*
gen rgdi = a261rx1q020sbea
gen rgdp = gdpc1
gen rgdo = sqrt(rgdi*rgdp)
cap drop gdo
* -------- set time periods -------------
global tq0 "1960q1"
global tq1 "1984q1"
global tq2 "2007q4"
global tqpm4 "2007q1"
global tqp "2008q1"
global tqpp1 "2008q2"
global tsfx "$tq1-$tqn"
global tqn "2018q1"
* ---------- Construct slack variables ---------- 
*
global bw1 = 60
local bw1 = $bw1
*
gen ugap = unrate - nrou
 label var ugap "Unemployment gap (CBO)"
gen gdpgap = 100*ln(gdpc1/gdppot)
 label var gdpgap "GDP gap (CBO)"
label var unrate "Unemployment rate"
label var unratel27 "Short-term un. rate"
gen emratio2554 = lns12300060
 label var emratio2554 "E-Pop, ages 25-54"
 *
global slackvars_raw "unrate unratel27 emratio emratio2554 tcu"
* Not real time gaps
global gapvars_cbo "ugap gdpgap"
global gapvars_bi ""
foreach y of varlist $slackvars_raw {
 tsbiweight_cea `y'_tr = `y', window(`bw1')
 gen `y'_bi = `y' - `y'_tr
 global gapvars_bi "$gapvars_bi `y'_bi"
}
dis "$gapvars_bi"
global slackvars_nrt "$gapvars_cbo $gapvars_bi"
dis "$slackvars_nrt"
*
* real time gaps, and nongapped variables
global slackvars_dm ""
local theta = 1 - 0.5^(1/60)
foreach y of varlist $slackvars_raw {
 tssmooth exponential `y'_sm = `y', parms(`theta')
 gen `y'_dm = `y' - `y'_sm
 global slackvars_dm "$slackvars_dm `y'_dm"
 corr `y' `y'_bi `y'_dm if tin($tq1,$tqn)
}
global slackvars_rt "unrate unratel27 $slackvars_dm"
dis "$slackvars_rt"
*
* standardized slack measures
su ugap if tin($tq1,$tqn)
 sca srtvars_sd = r(sd)
 sca srtvars_mn = r(mean)
 sca snrtvars_sd = r(sd)
 sca snrtvars_mn = r(mean)
global snrtvars "" 
foreach y in $slackvars_nrt {
 reg `y' unrate if tin($tq1,$tqn)
  sca ss = sign(_b[unrate])
 su `y' if tin($tq1,$tqn)
 gen s`y' = ss*(`y'-r(mean))*snrtvars_sd/r(sd) + snrtvars_mn
 global snrtvars "$snrtvars s`y'"
}
global srtvars "" 
foreach y in $slackvars_rt {
 reg `y' unrate if tin($tq1,$tqn)
  sca ss = sign(_b[unrate])
 su `y' if tin($tq1,$tqn)
 gen s`y' = ss*(`y'-r(mean))*srtvars_sd/r(sd) + srtvars_mn
 global srtvars "$srtvars s`y'"
}
foreach y in "emratio" "emratio2554" {
 reg `y' unrate if tin($tq1,$tqn)
  sca ss = sign(_b[unrate])
 su `y' if tin($tq1,$tqn)
 gen s`y' = ss*(`y'-r(mean))*srtvars_sd/r(sd) + srtvars_mn
}
* Extract principal components of gaps and cyclical activity variables
foreach vv in "snrtvars" "srtvars" {
 pca $`vv' if tin($tq1,$tqn), components(4)
  mat evec = e(L)
  mkmat $`vv', mat(xvarsmat)
  mat f = xvarsmat*evec
  cap drop f1-f4
  svmat f
*  su `vv'_stdvar if tin($tq1,$tqn)
*  reg f1 `vv'_stdvar if tin($tq1,$tqn)
*    sca ss = sign(_b[unrate])
  su f1 if tin($tq1,$tqn)
 gen pc1_`vv' = (f1-r(mean))*`vv'_sd/r(sd) + `vv'_mn
 global `vv' "pc1_`vv' $`vv'"
}
corr $snrtvars $srtvars if tin($tq1,$tqn)
*
global fpc1_snrtvars "Slack index (ex-post)"
global fsugap "Unemployment gap (CBO)"
global fsgdpgap "Output gap (CBO)"
global fsunrate_bi "Unemp. gap (2-sided filter)"
global fsunratel27_bi "Short-term un. gap (2-sided filter)"
global fsemratio_bi "Emp-pop ratio (2-sided filter)"
global fsemratio2554_bi "Emp-pop ages 25-54 (2-sided filter)"
global fstcu_bi "Cap. util. rate (2-sided filter)"
*
global fpc1_srtvars "Slack index"
global fsunrate "Unemployment rate"
global fsunratel27 "Short-term un. rate"
global fsunrate_dm "Unemployment gap (1-sided filter)"
global fsunratel27_dm "Short-term un. gap (1-sided filter)"
global fsemratio_dm "Emp-pop ratio (1-sided filter)"
global fsemratio2554_dm "Emp-pop ages 25-54 (1-sided filter)"
global fstcu_dm "Cap. util. rate (1-sided filter)"
*
global fpc1_sbpvars "Cyclical Activity Index"
global fslgdo_bw "GDO (cyclical component)"
global fslpayems_bw "Employment (cyclical component)"
global fsunrate_bw "Unemployment rate (cyclical component)"
global fsunratel27_bw "Short-term un. rate (cyclical component)"
global fstcu_bw "Capacity util. (cyclical component)"
global fsemratio_bw "Emp-pop ratio (cyclical component)"
*
foreach y in $snrtvars $srtvars $sbpvars {
 local fy "f`y'"
 label var `y' "$`fy'" 
}
* ---------- Inflation transformations and labels ---------- 
*
global cpi_indx "cpi_all cpi_xfe"
global agg_indx "pce_all pce_xfe pce_xe"  
global comp_indx "dmotrg dfdhrg dreqrg dodgrg dfxarg dclorg dgoerg dongrg p_hu_xe p_hu_es dhlcrg dtrsrg drcarg dfsarg difsrg dotsrg dnpirg "
global x4vars    "dmotrg dfdhrg        dodgrg dfxarg        dgoerg dongrg p_hu_xe p_hu_es dhlcrg dtrsrg drcarg dfsarg        dotsrg        "
global xevars "dmotrg dfdhrg dreqrg dodgrg dfxarg dclorg dongrg p_hu_xe dhlcrg dtrsrg drcarg dfsarg difsrg dotsrg dnpirg "
*
local fpce_all "PCE-All"
local fpce_xfe "PCE-xfe"
local fpce_xe "PCE-xe"
local fdmotrg "Motor vehicles & parts"
local fdfdhrg "Furnishings & durable household goods"
local fdreqrg "Recreational goods & vehicles"
local fdodgrg "Other durable goods"
local fdfxarg "Food & beverages off-premises"
local fdclorg "Clothing & footwear"	
local fdgoerg "Gasoline & energy goods"
local fdongrg "Other nondurable goods"
local fp_hu_xe = "Housing ex energy"
local fp_hu_es "Gas & electric util"
local fdhlcrg "Health care"
local fdtrsrg "Transportation services"
local fdrcarg "Recreation services"
local fdfsarg "Food services & accommodations"
local fdifsrg "Financial services & insurance"
local fdotsrg "Other services"
local fdnpirg "NPISH"
*
gen pce_xe_ucsvdevf = pce_xe - pce_xe_ucsvfil
gen pce_xe_ucsvdevs = pce_xe - pce_xe_ucsvsm
gen pce_xfe_ucsvdevf = pce_xfe - pce_xfe_ucsvfil
gen pce_xfe_ucsvdevs = pce_xfe - pce_xfe_ucsvsm
global ucsvvars "pce_xe_ucsvdevf pce_xe_ucsvdevs pce_xfe_ucsvdevf pce_xfe_ucsvdevs"
*
* inflation transformations ------------
sort time
foreach y in $ucsvvars $cpi_indx $comp_indx $agg_indx "ahe" {
 qui gen d`y' = `y'
 qui gen dd`y' = D.d`y'
 tsbiweight_cea dd`y'_tr = dd`y', window(`bw1')
  qui gen dd`y'_bi = dd`y' - dd`y'_tr
 qui gen d2`y' = (`y' + L.`y')/2
 qui gen d4`y' = (`y' + L.`y' + L2.`y' + L3.`y')/4
 qui gen d2d2`y' = d4`y' - L2.d4`y'
 qui gen d4d4`y' = d4`y' - L4.d4`y'
 qui gen d8d4`y' = d4`y' - L8.d4`y'
 tsbiweight_cea d`y'_tr = d`y', window(`bw1')
  qui gen d`y'_bi = d`y' - d`y'_tr
 cap drop b98
 tsfilter bw `y'_b = d`y', maxperiod(32) order(6) 
 tsfilter bw b98 = `y'_b, maxperiod(6) order(6) trend(`y'_bw)
}
* -------- Construct CSI using MWW wts (estimated in Matlab) --------
mat input csi_wt_mw = (0 0 0 0.15868 0 0 0.62852 0.0223 0 0 0.0855 0.03602 0.06897)
mat csi_wt_mw = csi_wt_mw'
mat list csi_wt_mw
gen d4d4csi = 0
gen d4csi = 0
gen csi = 0
local i = 0
foreach x in $x4vars {
 local i = `i'+1
 qui replace d4d4csi = d4d4csi + csi_wt_mw[`i',1]*d4d4`x'
 qui replace d4csi = d4csi + csi_wt_mw[`i',1]*d4`x'
 qui replace csi = csi + csi_wt_mw[`i',1]*`x'
}
*
label var d4csi "CSI"
label var d4pce_xfe "PCExFE"
* p_hu_xe:	Housing ex gas & electric util (inf@ar)
* dfxarg:	Food & beverages-off-premises (inf@ar)
* drcarg:	Recreation services (inf@ar)
list time d4pce_xfe d4p_hu_xe d4dfxarg d4drcarg if tin(2010q1,2018q1)
*
****************************************************************************
*         Sintra paper - Section 1
****************************************************************************
*
* ----- Chart 1.1 -----
label var d4pce_xfe "Core PCE"
label var d4pce_all "PCE"
twoway (tsline d4pce_xfe d4pce_all, lp(dash ..)) (tsline unrate, yaxis(2) lc(black)) if tin(2010q1,2018q1), ///
    legend(row(1) size(vsmall) rowgap(.05)) ytitle("4-qtr inflation")
  global a98 "figs\\Chart1-1.png"
  graph export $a98, replace
export excel year quarter d4pce_xfe d4pce_all unrate if tin(2010q1,2018q1) using "figs\Chart1-1.xlsx", firstrow(varl) replace
*
****************************************************************************
*         Sintra paper - Section 2
****************************************************************************
* ----- Chart 2.1 -----
label var tcu "Capacity utilization rate)
tsline $snrtvars if tin($tq1,$tqn), lp(solid dash ..) lw(medthick medthin ..) legend(col(2) size(vsmall) rowgap(.05)) 
  *title("Ex-post Gaps and Slack Index") note("Variables are transformed to have same mean, std. dev., and sign as the unemployment gap")
   global a98 "figs\Chart2-1.png"
   graph export $a98, replace
export excel year quarter $snrtvars if tin($tq1,$tqn) using "figs\Chart2-1.xlsx", firstrow(varl) replace
*
* ----- Charts 2.2, 2.3, and 3.7 ------
*
label var csi "CSI"
local tqnl $tqn
local ulag = 0
local malag = 3
local dd "d4d4"
foreach pp in "pce_xfe" "ahe" "csi" {
 if "`pp'" == "pce_xfe" {
  local fchart "Chart2-2"
  local xvlist "sugap sunratel27 stcu_bi sunrate"
 }
 if "`pp'" == "ahe" {
  local fchart "Chart2-3"
  local xvlist "sugap sunratel27"
 }
 if "`pp'" == "csi" {
  local fchart "Chart3-7"
  local xvlist "sugap sunratel27"
 }
 cap erase "data\\`fchart'.xlsx"
 local aa = 0
 foreach x in `xvlist' {
  local aa = `aa' + 1
  local faa = char(96+`aa')  
  cap drop dinf_6083 dinf_8499 dinf_0016 p6083 p8499 p0016 
  cap drop LMAx MAx 
  tssmooth ma MAx = `x', window(`malag' 1)
  gen LMAx = L`ulag'.MAx
  gen dinf_6083 = `dd'`pp' if tin(,1983q4)
  gen dinf_8499 = `dd'`pp' if tin(1984q1,1999q4)
  gen dinf_0016 = `dd'`pp' if tin(2000q1,`tqnl')
  reg `dd'`pp' LMAx if tin(1960q1,1983q1)
   predict p6083
   label var dinf_6083 "Inflation chge 60-83"
   label var p6083 "line 60-83"
  reg `dd'`pp' LMAx if tin(1984q1,1999q1)
   predict p8499
   label var dinf_8499 "Inflation chge 84-99"
   label var p8499 "line 84-99"
   reg `dd'`pp' LMAx if tin(2000q1,`tqnl')
   predict p0016
   label var dinf_0016 "Inflation chge 00-16"
   label var p0016 "line 00-16"
  sort LMAx
  local fx "f`x'"
  label var LMAx "$`fx'" 
  dis "$`fx'" 
  twoway scatter dinf_6083 p6083 dinf_8499 p8499 dinf_0016 p0016 LMAx if tin(1960q1,`tqnl'), ///
   ms(o i d i t i)  c(i l i l i l) legend(off) ysc(r(-5 5)) ylabel(-5(1)5) ///
   mc(blue = dkorange = dkgreen =) lc(blue = dkorange = dkgreen =) title("$`fx'") xtitle("") ytitle(`pp')
   global a98 "figs\\`fchart'`faa'.png"
   graph export $a98, replace
  export excel LMAx dinf_6083 p6083 dinf_8499 p8499 dinf_0016 p0016 using "figs\\`fchart'.xlsx", sheet("`fchart'`faa'") firstrow(varl) 
 sort time
 }
}
*
*-------------------------------------------------------------
*    Correlations and slope - over episodes - Table 2.1 & 2.3 & 3.4
*-------------------------------------------------------------
*
local dd "d4d4"
local ulag = 0
local malag = 3
local cvars ""
global tab21vars "sugap sgdpgap sunrate_bi sunratel27_bi semratio_bi semratio2554_bi stcu_bi pc1_snrtvars sunrate sunratel27" 
foreach pp in "pce_xfe" "ahe" "csi" {
 if "`pp'" == "pce_xfe" {
  global outxls "tables\Table2-1.xls"
 }
 if "`pp'" == "ahe" {
  global outxls "tables\Table2-3.xls"
 }
 if "`pp'" == "csi" {
  global outxls "tables\Table3-4.xls"
 }
  cap erase $outxls
  putexcel set $outxls, modify 
  putexcel A1=("PC regressions") ///
           A2=("Inflation:") B2=("`pp'") ///
           A3=("Transformation:") B3=("`dd'") ///
           A4=("MA order for x:") B4=("`malag'") ///
           A5=("Lag of x:") B5=("`ulag'") ///
           A6=("Controls") B6=("`cvars'") ///
           B8=("1960q1-1983q4") C8=("1984q1-1999q4") D8=("2000q1-2018q1") ///
           E8=("1960q1-1983q4") F8=("1984q1-1999q4") G8=("2000q1-2018q1") 
  local r = 7
   foreach x in $tab21vars {
   cap drop `x'_ma`malag'
   qui tssmooth ma `x'_ma`malag' = `x', window(`malag' 1)
   forvalues tp = 1/3 {
     if `tp' == 1 {
      local tq1 "1960q1"
      local tq2 "1983q4"
      }
     if `tp' == 2 {
      local tq1 "1984q1"
      local tq2 "1999q4"
      }
     if `tp' == 3 {
      local tq1 "2000q1"
      local tq2 "2018q4"
      }
    corr `dd'`pp' L`ulag'.`x'_ma`malag' if tin(`tq1',`tq2')
     local corr`tp' = string(r(rho),"%4.2f")
    newey `dd'`pp' L`ulag'.`x'_ma`malag' `cvars' if tin(`tq1',`tq2'), lag(8)
     local bhat`tp' = string(_b[L`ulag'.`x'_ma`malag'],"%4.2f")
     local se`tp' = "(" + string(_se[L`ulag'.`x'_ma`malag'],"%4.2f") + ")"
    }
  local r = `r' + 2
  local rp1 = `r' + 1
  putexcel A`r'=("`x'") B`r'=("`corr1'") C`r'=("`corr2'") D`r'=("`corr3'") ///
            E`r'=("`bhat1'") F`r'=("`bhat2'") G`r'=("`bhat3'") ///
            E`rp1'=("`se1'") F`rp1'=("`se2'") G`rp1'=("`se3'")
  }
}
*
*-------------------------------------------------------------
*    Cintra Table 2.2 & 2.4
*    Direct forecasting regressions 
*    (a) stability test post-84q1
*    (b) POOSE RMSE ratio, post-2013q4
*-------------------------------------------------------------
*
local dd "d4d4"
global tab22vars "sunrate sunratel27 sunrate_dm sunratel27_dm semratio_dm semratio2554_dm stcu_dm pc1_srtvars"
foreach pp in "pce_xfe" "ahe" {
 if "`pp'" == "pce_xfe" {
  global outxls "tables\Table2-2.xls"
 }
 if "`pp'" == "ahe" {
  global outxls "tables\Table2-4.xls"
 }
 cap erase $outxls
 putexcel set $outxls, modify 
 putexcel A1=("PC Forecasting regs: POOS and QLR") ///
          A2=("Inflation:") B2=("`pp'") ///
          A3=("Transformation:") B3=("`dd'") ///
          A4=("POOS Estimation") B4=("$tq1" + "-" + "$tqpm4") ///
          A5=("POOS period") B5=("$tqp" + "-" + "$tqn") ///
          B8=("SupW") C8=("POOS RMSFE ratio") 
 local r = 8
 foreach x in $tab22vars {
  * QLR test
  qlr `dd'`pp' L4.`x' L4.`dd'`pp' if tin($tq1,$tqn), newey(8)
   preserve
    clear
    use qlr
    qui su F
	sca qlrstat = r(max)
    local qlrstatf = string(qlrstat,"%5.2f")
   restore
  * ---- POOS - d4d4
  cap drop eu eu2 er er2
  * unrestricted 
  reg `dd'`pp' L4.`x' L4.`dd'`pp' if tin($tq1,$tqpm4)
    qui predict eu, resid
    qui gen eu2 = eu*eu
    qui su eu2 if tin($tqp,$tqn)
    sca mse_u = r(mean)
  * restricted
  reg `dd'`pp' L4.`dd'`pp' if tin($tq1,$tqpm4)
   qui predict er, resid
    qui gen er2 = er*er
    qui su er2 if tin($tqp,$tqn)
    sca mse_r = r(mean)
    sca rmserat = sqrt(mse_u/mse_r)
    local rmseratf = string(rmserat,"%5.3f")
  * ----- Save results ------
  local r = `r' + 1
  putexcel A`r'=("`x'") B`r'=("`qlrstatf'") C`r'=("`rmseratf'") 
 }
}
*
